# frozen_string_literal: true

# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Auto-generated by gapic-generator-ruby. DO NOT EDIT!

require "google/cloud/errors"
require "google/cloud/apigateway/v1/apigateway_service_pb"

module Google
  module Cloud
    module ApiGateway
      module V1
        module ApiGatewayService
          ##
          # Client for the ApiGatewayService service.
          #
          # The API Gateway Service is the interface for managing API Gateways.
          #
          class Client
            include Paths

            # @private
            attr_reader :api_gateway_service_stub

            ##
            # Configure the ApiGatewayService Client class.
            #
            # See {::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client::Configuration}
            # for a description of the configuration fields.
            #
            # @example
            #
            #   # Modify the configuration for all ApiGatewayService clients
            #   ::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client.configure do |config|
            #     config.timeout = 10.0
            #   end
            #
            # @yield [config] Configure the Client client.
            # @yieldparam config [Client::Configuration]
            #
            # @return [Client::Configuration]
            #
            def self.configure
              @configure ||= begin
                namespace = ["Google", "Cloud", "ApiGateway", "V1"]
                parent_config = while namespace.any?
                                  parent_name = namespace.join "::"
                                  parent_const = const_get parent_name
                                  break parent_const.configure if parent_const.respond_to? :configure
                                  namespace.pop
                                end
                default_config = Client::Configuration.new parent_config

                default_config.rpcs.create_gateway.timeout = 60.0
                default_config.rpcs.create_gateway.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.update_gateway.timeout = 60.0
                default_config.rpcs.update_gateway.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.delete_gateway.timeout = 60.0
                default_config.rpcs.delete_gateway.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.create_api.timeout = 60.0
                default_config.rpcs.create_api.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.update_api.timeout = 60.0
                default_config.rpcs.update_api.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.delete_api.timeout = 60.0
                default_config.rpcs.delete_api.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.create_api_config.timeout = 60.0
                default_config.rpcs.create_api_config.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.update_api_config.timeout = 60.0
                default_config.rpcs.update_api_config.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config.rpcs.delete_api_config.timeout = 60.0
                default_config.rpcs.delete_api_config.retry_policy = {
                  initial_delay: 1.0, max_delay: 60.0, multiplier: 2, retry_codes: [2, 14]
                }

                default_config
              end
              yield @configure if block_given?
              @configure
            end

            ##
            # Configure the ApiGatewayService Client instance.
            #
            # The configuration is set to the derived mode, meaning that values can be changed,
            # but structural changes (adding new fields, etc.) are not allowed. Structural changes
            # should be made on {Client.configure}.
            #
            # See {::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client::Configuration}
            # for a description of the configuration fields.
            #
            # @yield [config] Configure the Client client.
            # @yieldparam config [Client::Configuration]
            #
            # @return [Client::Configuration]
            #
            def configure
              yield @config if block_given?
              @config
            end

            ##
            # Create a new ApiGatewayService client object.
            #
            # @example
            #
            #   # Create a client using the default configuration
            #   client = ::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client.new
            #
            #   # Create a client using a custom configuration
            #   client = ::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client.new do |config|
            #     config.timeout = 10.0
            #   end
            #
            # @yield [config] Configure the ApiGatewayService client.
            # @yieldparam config [Client::Configuration]
            #
            def initialize
              # These require statements are intentionally placed here to initialize
              # the gRPC module only when it's required.
              # See https://github.com/googleapis/toolkit/issues/446
              require "gapic/grpc"
              require "google/cloud/apigateway/v1/apigateway_service_services_pb"

              # Create the configuration object
              @config = Configuration.new Client.configure

              # Yield the configuration if needed
              yield @config if block_given?

              # Create credentials
              credentials = @config.credentials
              # Use self-signed JWT if the endpoint is unchanged from default,
              # but only if the default endpoint does not have a region prefix.
              enable_self_signed_jwt = @config.endpoint == Client.configure.endpoint &&
                                       !@config.endpoint.split(".").first.include?("-")
              credentials ||= Credentials.default scope: @config.scope,
                                                  enable_self_signed_jwt: enable_self_signed_jwt
              if credentials.is_a?(::String) || credentials.is_a?(::Hash)
                credentials = Credentials.new credentials, scope: @config.scope
              end
              @quota_project_id = @config.quota_project
              @quota_project_id ||= credentials.quota_project_id if credentials.respond_to? :quota_project_id

              @operations_client = Operations.new do |config|
                config.credentials = credentials
                config.endpoint = @config.endpoint
              end

              @api_gateway_service_stub = ::Gapic::ServiceStub.new(
                ::Google::Cloud::ApiGateway::V1::ApiGatewayService::Stub,
                credentials:  credentials,
                endpoint:     @config.endpoint,
                channel_args: @config.channel_args,
                interceptors: @config.interceptors
              )
            end

            ##
            # Get the associated client for long-running operations.
            #
            # @return [::Google::Cloud::ApiGateway::V1::ApiGatewayService::Operations]
            #
            attr_reader :operations_client

            # Service calls

            ##
            # Lists Gateways in a given project and location.
            #
            # @overload list_gateways(request, options = nil)
            #   Pass arguments to `list_gateways` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::ListGatewaysRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::ListGatewaysRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload list_gateways(parent: nil, page_size: nil, page_token: nil, filter: nil, order_by: nil)
            #   Pass arguments to `list_gateways` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. Parent resource of the Gateway, of the form:
            #     `projects/*/locations/*`
            #   @param page_size [::Integer]
            #     Page size.
            #   @param page_token [::String]
            #     Page token.
            #   @param filter [::String]
            #     Filter.
            #   @param order_by [::String]
            #     Order by parameters.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::PagedEnumerable<::Google::Cloud::ApiGateway::V1::Gateway>]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::PagedEnumerable<::Google::Cloud::ApiGateway::V1::Gateway>]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def list_gateways request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::ListGatewaysRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.list_gateways.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.list_gateways.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.list_gateways.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :list_gateways, request, options: options do |response, operation|
                response = ::Gapic::PagedEnumerable.new @api_gateway_service_stub, :list_gateways, request, response, operation, options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Gets details of a single Gateway.
            #
            # @overload get_gateway(request, options = nil)
            #   Pass arguments to `get_gateway` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::GetGatewayRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::GetGatewayRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload get_gateway(name: nil)
            #   Pass arguments to `get_gateway` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Resource name of the form:
            #     `projects/*/locations/*/gateways/*`
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Cloud::ApiGateway::V1::Gateway]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Cloud::ApiGateway::V1::Gateway]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def get_gateway request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::GetGatewayRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.get_gateway.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.get_gateway.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.get_gateway.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :get_gateway, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Creates a new Gateway in a given project and location.
            #
            # @overload create_gateway(request, options = nil)
            #   Pass arguments to `create_gateway` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::CreateGatewayRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::CreateGatewayRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload create_gateway(parent: nil, gateway_id: nil, gateway: nil)
            #   Pass arguments to `create_gateway` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. Parent resource of the Gateway, of the form:
            #     `projects/*/locations/*`
            #   @param gateway_id [::String]
            #     Required. Identifier to assign to the Gateway. Must be unique within scope of
            #     the parent resource.
            #   @param gateway [::Google::Cloud::ApiGateway::V1::Gateway, ::Hash]
            #     Required. Gateway resource.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def create_gateway request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::CreateGatewayRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.create_gateway.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.create_gateway.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.create_gateway.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :create_gateway, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Updates the parameters of a single Gateway.
            #
            # @overload update_gateway(request, options = nil)
            #   Pass arguments to `update_gateway` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::UpdateGatewayRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::UpdateGatewayRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload update_gateway(update_mask: nil, gateway: nil)
            #   Pass arguments to `update_gateway` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param update_mask [::Google::Protobuf::FieldMask, ::Hash]
            #     Field mask is used to specify the fields to be overwritten in the
            #     Gateway resource by the update.
            #     The fields specified in the update_mask are relative to the resource, not
            #     the full request. A field will be overwritten if it is in the mask. If the
            #     user does not provide a mask then all fields will be overwritten.
            #   @param gateway [::Google::Cloud::ApiGateway::V1::Gateway, ::Hash]
            #     Required. Gateway resource.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def update_gateway request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::UpdateGatewayRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.update_gateway.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "gateway.name" => request.gateway.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.update_gateway.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.update_gateway.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :update_gateway, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Deletes a single Gateway.
            #
            # @overload delete_gateway(request, options = nil)
            #   Pass arguments to `delete_gateway` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::DeleteGatewayRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::DeleteGatewayRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload delete_gateway(name: nil)
            #   Pass arguments to `delete_gateway` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Resource name of the form:
            #     `projects/*/locations/*/gateways/*`
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def delete_gateway request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::DeleteGatewayRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.delete_gateway.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.delete_gateway.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.delete_gateway.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :delete_gateway, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Lists Apis in a given project and location.
            #
            # @overload list_apis(request, options = nil)
            #   Pass arguments to `list_apis` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::ListApisRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::ListApisRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload list_apis(parent: nil, page_size: nil, page_token: nil, filter: nil, order_by: nil)
            #   Pass arguments to `list_apis` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. Parent resource of the API, of the form:
            #     `projects/*/locations/global`
            #   @param page_size [::Integer]
            #     Page size.
            #   @param page_token [::String]
            #     Page token.
            #   @param filter [::String]
            #     Filter.
            #   @param order_by [::String]
            #     Order by parameters.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::PagedEnumerable<::Google::Cloud::ApiGateway::V1::Api>]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::PagedEnumerable<::Google::Cloud::ApiGateway::V1::Api>]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def list_apis request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::ListApisRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.list_apis.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.list_apis.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.list_apis.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :list_apis, request, options: options do |response, operation|
                response = ::Gapic::PagedEnumerable.new @api_gateway_service_stub, :list_apis, request, response, operation, options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Gets details of a single Api.
            #
            # @overload get_api(request, options = nil)
            #   Pass arguments to `get_api` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::GetApiRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::GetApiRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload get_api(name: nil)
            #   Pass arguments to `get_api` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Resource name of the form:
            #     `projects/*/locations/global/apis/*`
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Cloud::ApiGateway::V1::Api]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Cloud::ApiGateway::V1::Api]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def get_api request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::GetApiRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.get_api.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.get_api.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.get_api.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :get_api, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Creates a new Api in a given project and location.
            #
            # @overload create_api(request, options = nil)
            #   Pass arguments to `create_api` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::CreateApiRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::CreateApiRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload create_api(parent: nil, api_id: nil, api: nil)
            #   Pass arguments to `create_api` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. Parent resource of the API, of the form:
            #     `projects/*/locations/global`
            #   @param api_id [::String]
            #     Required. Identifier to assign to the API. Must be unique within scope of
            #     the parent resource.
            #   @param api [::Google::Cloud::ApiGateway::V1::Api, ::Hash]
            #     Required. API resource.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def create_api request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::CreateApiRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.create_api.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.create_api.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.create_api.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :create_api, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Updates the parameters of a single Api.
            #
            # @overload update_api(request, options = nil)
            #   Pass arguments to `update_api` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::UpdateApiRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::UpdateApiRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload update_api(update_mask: nil, api: nil)
            #   Pass arguments to `update_api` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param update_mask [::Google::Protobuf::FieldMask, ::Hash]
            #     Field mask is used to specify the fields to be overwritten in the
            #     Api resource by the update.
            #     The fields specified in the update_mask are relative to the resource, not
            #     the full request. A field will be overwritten if it is in the mask. If the
            #     user does not provide a mask then all fields will be overwritten.
            #   @param api [::Google::Cloud::ApiGateway::V1::Api, ::Hash]
            #     Required. API resource.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def update_api request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::UpdateApiRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.update_api.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "api.name" => request.api.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.update_api.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.update_api.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :update_api, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Deletes a single Api.
            #
            # @overload delete_api(request, options = nil)
            #   Pass arguments to `delete_api` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::DeleteApiRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::DeleteApiRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload delete_api(name: nil)
            #   Pass arguments to `delete_api` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Resource name of the form:
            #     `projects/*/locations/global/apis/*`
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def delete_api request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::DeleteApiRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.delete_api.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.delete_api.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.delete_api.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :delete_api, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Lists ApiConfigs in a given project and location.
            #
            # @overload list_api_configs(request, options = nil)
            #   Pass arguments to `list_api_configs` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::ListApiConfigsRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::ListApiConfigsRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload list_api_configs(parent: nil, page_size: nil, page_token: nil, filter: nil, order_by: nil)
            #   Pass arguments to `list_api_configs` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. Parent resource of the API Config, of the form:
            #     `projects/*/locations/global/apis/*`
            #   @param page_size [::Integer]
            #     Page size.
            #   @param page_token [::String]
            #     Page token.
            #   @param filter [::String]
            #     Filter.
            #   @param order_by [::String]
            #     Order by parameters.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::PagedEnumerable<::Google::Cloud::ApiGateway::V1::ApiConfig>]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::PagedEnumerable<::Google::Cloud::ApiGateway::V1::ApiConfig>]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def list_api_configs request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::ListApiConfigsRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.list_api_configs.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.list_api_configs.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.list_api_configs.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :list_api_configs, request, options: options do |response, operation|
                response = ::Gapic::PagedEnumerable.new @api_gateway_service_stub, :list_api_configs, request, response, operation, options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Gets details of a single ApiConfig.
            #
            # @overload get_api_config(request, options = nil)
            #   Pass arguments to `get_api_config` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::GetApiConfigRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::GetApiConfigRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload get_api_config(name: nil, view: nil)
            #   Pass arguments to `get_api_config` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Resource name of the form:
            #     `projects/*/locations/global/apis/*/configs/*`
            #   @param view [::Google::Cloud::ApiGateway::V1::GetApiConfigRequest::ConfigView]
            #     Specifies which fields of the API Config are returned in the response.
            #     Defaults to `BASIC` view.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Cloud::ApiGateway::V1::ApiConfig]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Cloud::ApiGateway::V1::ApiConfig]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def get_api_config request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::GetApiConfigRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.get_api_config.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.get_api_config.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.get_api_config.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :get_api_config, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Creates a new ApiConfig in a given project and location.
            #
            # @overload create_api_config(request, options = nil)
            #   Pass arguments to `create_api_config` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::CreateApiConfigRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::CreateApiConfigRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload create_api_config(parent: nil, api_config_id: nil, api_config: nil)
            #   Pass arguments to `create_api_config` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. Parent resource of the API Config, of the form:
            #     `projects/*/locations/global/apis/*`
            #   @param api_config_id [::String]
            #     Required. Identifier to assign to the API Config. Must be unique within scope of
            #     the parent resource.
            #   @param api_config [::Google::Cloud::ApiGateway::V1::ApiConfig, ::Hash]
            #     Required. API resource.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def create_api_config request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::CreateApiConfigRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.create_api_config.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.create_api_config.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.create_api_config.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :create_api_config, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Updates the parameters of a single ApiConfig.
            #
            # @overload update_api_config(request, options = nil)
            #   Pass arguments to `update_api_config` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::UpdateApiConfigRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::UpdateApiConfigRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload update_api_config(update_mask: nil, api_config: nil)
            #   Pass arguments to `update_api_config` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param update_mask [::Google::Protobuf::FieldMask, ::Hash]
            #     Field mask is used to specify the fields to be overwritten in the
            #     ApiConfig resource by the update.
            #     The fields specified in the update_mask are relative to the resource, not
            #     the full request. A field will be overwritten if it is in the mask. If the
            #     user does not provide a mask then all fields will be overwritten.
            #   @param api_config [::Google::Cloud::ApiGateway::V1::ApiConfig, ::Hash]
            #     Required. API Config resource.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def update_api_config request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::UpdateApiConfigRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.update_api_config.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "api_config.name" => request.api_config.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.update_api_config.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.update_api_config.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :update_api_config, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Deletes a single ApiConfig.
            #
            # @overload delete_api_config(request, options = nil)
            #   Pass arguments to `delete_api_config` via a request object, either of type
            #   {::Google::Cloud::ApiGateway::V1::DeleteApiConfigRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::ApiGateway::V1::DeleteApiConfigRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload delete_api_config(name: nil)
            #   Pass arguments to `delete_api_config` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Resource name of the form:
            #     `projects/*/locations/global/apis/*/configs/*`
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::Operation]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::Operation]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def delete_api_config request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::ApiGateway::V1::DeleteApiConfigRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.delete_api_config.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::ApiGateway::V1::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.delete_api_config.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.delete_api_config.retry_policy

              options.apply_defaults timeout:      @config.timeout,
                                     metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @api_gateway_service_stub.call_rpc :delete_api_config, request, options: options do |response, operation|
                response = ::Gapic::Operation.new response, @operations_client, options: options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Configuration class for the ApiGatewayService API.
            #
            # This class represents the configuration for ApiGatewayService,
            # providing control over timeouts, retry behavior, logging, transport
            # parameters, and other low-level controls. Certain parameters can also be
            # applied individually to specific RPCs. See
            # {::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client::Configuration::Rpcs}
            # for a list of RPCs that can be configured independently.
            #
            # Configuration can be applied globally to all clients, or to a single client
            # on construction.
            #
            # @example
            #
            #   # Modify the global config, setting the timeout for
            #   # list_gateways to 20 seconds,
            #   # and all remaining timeouts to 10 seconds.
            #   ::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client.configure do |config|
            #     config.timeout = 10.0
            #     config.rpcs.list_gateways.timeout = 20.0
            #   end
            #
            #   # Apply the above configuration only to a new client.
            #   client = ::Google::Cloud::ApiGateway::V1::ApiGatewayService::Client.new do |config|
            #     config.timeout = 10.0
            #     config.rpcs.list_gateways.timeout = 20.0
            #   end
            #
            # @!attribute [rw] endpoint
            #   The hostname or hostname:port of the service endpoint.
            #   Defaults to `"apigateway.googleapis.com"`.
            #   @return [::String]
            # @!attribute [rw] credentials
            #   Credentials to send with calls. You may provide any of the following types:
            #    *  (`String`) The path to a service account key file in JSON format
            #    *  (`Hash`) A service account key as a Hash
            #    *  (`Google::Auth::Credentials`) A googleauth credentials object
            #       (see the [googleauth docs](https://googleapis.dev/ruby/googleauth/latest/index.html))
            #    *  (`Signet::OAuth2::Client`) A signet oauth2 client object
            #       (see the [signet docs](https://googleapis.dev/ruby/signet/latest/Signet/OAuth2/Client.html))
            #    *  (`GRPC::Core::Channel`) a gRPC channel with included credentials
            #    *  (`GRPC::Core::ChannelCredentials`) a gRPC credentails object
            #    *  (`nil`) indicating no credentials
            #   @return [::Object]
            # @!attribute [rw] scope
            #   The OAuth scopes
            #   @return [::Array<::String>]
            # @!attribute [rw] lib_name
            #   The library name as recorded in instrumentation and logging
            #   @return [::String]
            # @!attribute [rw] lib_version
            #   The library version as recorded in instrumentation and logging
            #   @return [::String]
            # @!attribute [rw] channel_args
            #   Extra parameters passed to the gRPC channel. Note: this is ignored if a
            #   `GRPC::Core::Channel` object is provided as the credential.
            #   @return [::Hash]
            # @!attribute [rw] interceptors
            #   An array of interceptors that are run before calls are executed.
            #   @return [::Array<::GRPC::ClientInterceptor>]
            # @!attribute [rw] timeout
            #   The call timeout in seconds.
            #   @return [::Numeric]
            # @!attribute [rw] metadata
            #   Additional gRPC headers to be sent with the call.
            #   @return [::Hash{::Symbol=>::String}]
            # @!attribute [rw] retry_policy
            #   The retry policy. The value is a hash with the following keys:
            #    *  `:initial_delay` (*type:* `Numeric`) - The initial delay in seconds.
            #    *  `:max_delay` (*type:* `Numeric`) - The max delay in seconds.
            #    *  `:multiplier` (*type:* `Numeric`) - The incremental backoff multiplier.
            #    *  `:retry_codes` (*type:* `Array<String>`) - The error codes that should
            #       trigger a retry.
            #   @return [::Hash]
            # @!attribute [rw] quota_project
            #   A separate project against which to charge quota.
            #   @return [::String]
            #
            class Configuration
              extend ::Gapic::Config

              config_attr :endpoint,      "apigateway.googleapis.com", ::String
              config_attr :credentials,   nil do |value|
                allowed = [::String, ::Hash, ::Proc, ::Symbol, ::Google::Auth::Credentials, ::Signet::OAuth2::Client, nil]
                allowed += [::GRPC::Core::Channel, ::GRPC::Core::ChannelCredentials] if defined? ::GRPC
                allowed.any? { |klass| klass === value }
              end
              config_attr :scope,         nil, ::String, ::Array, nil
              config_attr :lib_name,      nil, ::String, nil
              config_attr :lib_version,   nil, ::String, nil
              config_attr(:channel_args,  { "grpc.service_config_disable_resolution" => 1 }, ::Hash, nil)
              config_attr :interceptors,  nil, ::Array, nil
              config_attr :timeout,       nil, ::Numeric, nil
              config_attr :metadata,      nil, ::Hash, nil
              config_attr :retry_policy,  nil, ::Hash, ::Proc, nil
              config_attr :quota_project, nil, ::String, nil

              # @private
              def initialize parent_config = nil
                @parent_config = parent_config unless parent_config.nil?

                yield self if block_given?
              end

              ##
              # Configurations for individual RPCs
              # @return [Rpcs]
              #
              def rpcs
                @rpcs ||= begin
                  parent_rpcs = nil
                  parent_rpcs = @parent_config.rpcs if defined?(@parent_config) && @parent_config.respond_to?(:rpcs)
                  Rpcs.new parent_rpcs
                end
              end

              ##
              # Configuration RPC class for the ApiGatewayService API.
              #
              # Includes fields providing the configuration for each RPC in this service.
              # Each configuration object is of type `Gapic::Config::Method` and includes
              # the following configuration fields:
              #
              #  *  `timeout` (*type:* `Numeric`) - The call timeout in seconds
              #  *  `metadata` (*type:* `Hash{Symbol=>String}`) - Additional gRPC headers
              #  *  `retry_policy (*type:* `Hash`) - The retry policy. The policy fields
              #     include the following keys:
              #      *  `:initial_delay` (*type:* `Numeric`) - The initial delay in seconds.
              #      *  `:max_delay` (*type:* `Numeric`) - The max delay in seconds.
              #      *  `:multiplier` (*type:* `Numeric`) - The incremental backoff multiplier.
              #      *  `:retry_codes` (*type:* `Array<String>`) - The error codes that should
              #         trigger a retry.
              #
              class Rpcs
                ##
                # RPC-specific configuration for `list_gateways`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :list_gateways
                ##
                # RPC-specific configuration for `get_gateway`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :get_gateway
                ##
                # RPC-specific configuration for `create_gateway`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :create_gateway
                ##
                # RPC-specific configuration for `update_gateway`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :update_gateway
                ##
                # RPC-specific configuration for `delete_gateway`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :delete_gateway
                ##
                # RPC-specific configuration for `list_apis`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :list_apis
                ##
                # RPC-specific configuration for `get_api`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :get_api
                ##
                # RPC-specific configuration for `create_api`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :create_api
                ##
                # RPC-specific configuration for `update_api`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :update_api
                ##
                # RPC-specific configuration for `delete_api`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :delete_api
                ##
                # RPC-specific configuration for `list_api_configs`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :list_api_configs
                ##
                # RPC-specific configuration for `get_api_config`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :get_api_config
                ##
                # RPC-specific configuration for `create_api_config`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :create_api_config
                ##
                # RPC-specific configuration for `update_api_config`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :update_api_config
                ##
                # RPC-specific configuration for `delete_api_config`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :delete_api_config

                # @private
                def initialize parent_rpcs = nil
                  list_gateways_config = parent_rpcs.list_gateways if parent_rpcs.respond_to? :list_gateways
                  @list_gateways = ::Gapic::Config::Method.new list_gateways_config
                  get_gateway_config = parent_rpcs.get_gateway if parent_rpcs.respond_to? :get_gateway
                  @get_gateway = ::Gapic::Config::Method.new get_gateway_config
                  create_gateway_config = parent_rpcs.create_gateway if parent_rpcs.respond_to? :create_gateway
                  @create_gateway = ::Gapic::Config::Method.new create_gateway_config
                  update_gateway_config = parent_rpcs.update_gateway if parent_rpcs.respond_to? :update_gateway
                  @update_gateway = ::Gapic::Config::Method.new update_gateway_config
                  delete_gateway_config = parent_rpcs.delete_gateway if parent_rpcs.respond_to? :delete_gateway
                  @delete_gateway = ::Gapic::Config::Method.new delete_gateway_config
                  list_apis_config = parent_rpcs.list_apis if parent_rpcs.respond_to? :list_apis
                  @list_apis = ::Gapic::Config::Method.new list_apis_config
                  get_api_config = parent_rpcs.get_api if parent_rpcs.respond_to? :get_api
                  @get_api = ::Gapic::Config::Method.new get_api_config
                  create_api_config = parent_rpcs.create_api if parent_rpcs.respond_to? :create_api
                  @create_api = ::Gapic::Config::Method.new create_api_config
                  update_api_config = parent_rpcs.update_api if parent_rpcs.respond_to? :update_api
                  @update_api = ::Gapic::Config::Method.new update_api_config
                  delete_api_config = parent_rpcs.delete_api if parent_rpcs.respond_to? :delete_api
                  @delete_api = ::Gapic::Config::Method.new delete_api_config
                  list_api_configs_config = parent_rpcs.list_api_configs if parent_rpcs.respond_to? :list_api_configs
                  @list_api_configs = ::Gapic::Config::Method.new list_api_configs_config
                  get_api_config_config = parent_rpcs.get_api_config if parent_rpcs.respond_to? :get_api_config
                  @get_api_config = ::Gapic::Config::Method.new get_api_config_config
                  create_api_config_config = parent_rpcs.create_api_config if parent_rpcs.respond_to? :create_api_config
                  @create_api_config = ::Gapic::Config::Method.new create_api_config_config
                  update_api_config_config = parent_rpcs.update_api_config if parent_rpcs.respond_to? :update_api_config
                  @update_api_config = ::Gapic::Config::Method.new update_api_config_config
                  delete_api_config_config = parent_rpcs.delete_api_config if parent_rpcs.respond_to? :delete_api_config
                  @delete_api_config = ::Gapic::Config::Method.new delete_api_config_config

                  yield self if block_given?
                end
              end
            end
          end
        end
      end
    end
  end
end
